@@ -19,5 +19,9 @@ main(int argc, char *argv[]) {
 	perror("VT_ACTIVATE");
 	exit(1);
     }
+    if (ioctl(fd,VT_WAITACTIVE,num)) {
+	perror("VT_WAITACTIVE");
+	exit(1);
+    }
     exit(0);
 }
