@@ -12,6 +12,7 @@
  * for 0.86: with -l option: also tell about synonyms
  * for 0.87: output "charset iso-8859-x" so that loadkeys
  *      can handle the output of dumpkeys -c
+ * for 0.88: handle sparse keymaps
  */
 #include <stdio.h>
 #include <ctype.h>
@@ -22,15 +23,75 @@
 #include <linux/keyboard.h>
 #include <sys/ioctl.h>
 #include <string.h>
+#include <errno.h>
 #include "ksyms.h"
 
 #ifndef KT_LETTER
 #define KT_LETTER KT_LATIN
 #endif
 
-#define VERSION "0.87"
+#ifndef MAX_NR_KEYMAPS
+#define MAX_NR_KEYMAPS NR_KEYMAPS
+#endif
+
+#define VERSION "0.88"
 static int fd;
 
+char keymap_exists[MAX_NR_KEYMAPS];
+int good_keymap[MAX_NR_KEYMAPS], keymapnr, allocct;
+
+void get_keymaps(void) {
+	int i, j;
+	struct kbentry ke;
+
+	keymapnr = allocct = 0;
+	for (i=0; i<MAX_NR_KEYMAPS; i++) {
+	    ke.kb_index = 0;
+	    ke.kb_table = i;
+	    j = ioctl(fd, KDGKBENT, (unsigned long)&ke);
+	    if (j && errno != EINVAL) {
+		fprintf(stderr, "KDGKBENT at index 0 in table %d: ", i);
+		perror("");
+		exit(1);
+	    }
+	    if (!j && ke.kb_value != K_NOSUCHMAP) {
+		keymap_exists[i] = 1;
+		good_keymap[keymapnr++] = i;
+		if (ke.kb_value == K_ALLOCATED)
+		  allocct++;
+	    } else {
+		keymap_exists[i] = 0;
+	    }
+	}
+	if (keymapnr == 0) {
+	    fprintf(stderr, "dumpkeys: cannot find any keymaps?\n");
+	    exit(1);
+	}
+	if (good_keymap[0] != 0) {
+	    fprintf(stderr,
+		    "dumpkeys: plain map not allocated? very strange ...\n");
+	    /* this is not fatal */
+	}
+}
+
+void print_keymaps(void) {
+	int i,m0,m;
+
+	printf("keymaps ");
+	for (i=0; i<keymapnr; i++) {
+	    if (i)
+	      printf(",");
+	    m0 = m = good_keymap[i];
+	    while (i+1 < keymapnr && good_keymap[i+1] == m+1)
+	      i++, m++;
+	    if (m0 == m)
+	      printf("%d", m0);
+	    else
+	      printf("%d-%d", m0, m);
+	}
+	printf("\n");
+}
+
 int get_bind(u_char index, u_char table) {
 	struct kbentry ke;
 
@@ -53,6 +114,10 @@ void print_keysym(int code, char numeric) {
 	printf(" ");
 	t = KTYP(code);
 	v = KVAL(code);
+	if (t > KT_LETTER) {
+	        printf("U%04x          ", code ^ 0xf000);
+		return;
+	}
 	if (t == KT_LETTER) {
 	        t = KT_LATIN;
 		    printf("+");
@@ -68,25 +133,17 @@ void print_keysym(int code, char numeric) {
 		printf("0x%04x          ", code);
 }
 
-/* there's a race in here - we might destroy a change */
 char valid_type(int t) {
 	struct kbentry ke;
-	int old_value;
 	char status;
 
 	ke.kb_index = 0;
 	ke.kb_table = 0;
-	if (ioctl(fd, KDGKBENT, (unsigned long)&ke))
-		return 0;
-	old_value = ke.kb_value;
 	ke.kb_value = K(t, 0);
-	status = ioctl(fd, KDSKBENT, (unsigned long)&ke) == 0;
-	ke.kb_value = old_value;
-	ioctl(fd, KDSKBENT, (unsigned long)&ke);
+	status = (ioctl(fd, KDSKBENT, (unsigned long)&ke) == 0);
 	return status;
 }
 
-/* there's a race in here */
 u_char maximum_val(int t) {
 	struct kbentry ke;
 	int old_value;
@@ -94,41 +151,57 @@ u_char maximum_val(int t) {
 
 	ke.kb_index = 0;
 	ke.kb_table = 0;
-	if (ioctl(fd, KDGKBENT, (unsigned long)&ke))
-		return 0;
-	old_value = ke.kb_value;
 
 	for (i = 0; i < 256; i++) {
 		ke.kb_value = K(t, i);
 		if (ioctl(fd, KDSKBENT, (unsigned long)&ke))
 			break;
 	}
-
-	ke.kb_value = old_value;
-	ioctl(fd, KDSKBENT, (unsigned long)&ke);
+	ke.kb_value = K_HOLE;
+	ioctl(fd, KDSKBENT, (unsigned long)&ke); 	/* superfluous */
 
 	return i - 1;
 }
 
+#define NR_TYPES 12
+int maxval[NR_TYPES];
 
 void show_short_info(void) {
 	int i;
 
-	printf("keycode range supported by kernel:           0 - %d\n",
-		NR_KEYS - 1);
-	printf("number of actions bindable to a key:         %d\n",
-		NR_KEYMAPS);
+	printf("keycode range supported by kernel:           1 - %d\n",
+	       NR_KEYS - 1);
+	printf("max number of actions bindable to a key:         %d\n",
+	       MAX_NR_KEYMAPS);
+	get_keymaps();
+	printf("number of keymaps in actual use:                 %d\n",
+	       keymapnr);
+	if (allocct)
+	  printf("of which %d dynamically allocated\n", allocct);
 	printf("ranges of action codes supported by kernel:\n");
-	for (i = 0; valid_type(i); i++)
-		printf("	0x%04x - 0x%04x\n",
-			K(i, 0), K(i, maximum_val(i)));
-	printf("number of function keys supported by kernel: %d\n", NR_FUNC);
-	printf("total space available for function strings:  %d bytes\n",
-		FUNC_BUFSIZE);
+	for (i = 0; i < NR_TYPES && valid_type(i); i++) {
+	    maxval[i] = maximum_val(i);
+	    printf("	0x%04x - 0x%04x\n", K(i, 0), K(i, maxval[i]));
+	}
+	printf("number of function keys supported by kernel: %d\n", MAX_NR_FUNC);
 	printf("total space available for compose key definitions: %d bytes\n",
 	        MAX_DIACR);
 }
 
+struct {
+    char *name;
+    int bit;
+} modifiers[] = {
+    { "shift",	KG_SHIFT  },
+    { "altgr",	KG_ALTGR  },
+    { "control",KG_CTRL   },
+    { "alt",	KG_ALT    },
+    { "shiftl",	KG_SHIFTL },
+    { "shiftr",	KG_SHIFTR },
+    { "ctrll",	KG_CTRLL  },
+    { "ctrlr",	KG_CTRLR  }
+};
+
 void dump_symbols(void) {
 	int t;
 	int v;
@@ -141,124 +214,115 @@ void dump_symbols(void) {
 				printf("0x%04x\t%s\n", K(t, v), p);
 	printf("\nThe following synonyms are recognized:\n\n");
 	for (t = 0; t < syn_size; t++)
-	    printf("%-15s for %s\n", synonyms[t].synonym,
-		   synonyms[t].official_name);
-	printf("
-Recognized modifier names and their column numbers:
-
-shift	%d
-altgr	%d
-control	%d
-alt	%d
-", (1 << KG_SHIFT), (1 << KG_ALTGR), (1 << KG_CTRL), (1 << KG_ALT));
+	  printf("%-15s for %s\n", synonyms[t].synonym,
+		 synonyms[t].official_name);
+	printf("\nRecognized modifier names and their column numbers:\n");
+	for (t = 0; t < sizeof(modifiers)/sizeof(modifiers[0]); t++)
+	  printf("%s\t%d", modifiers[t].name, 1 << modifiers[t].bit);
 }
 
 void print_mod(int x) {
-	if (x & (1 << KG_SHIFT))
-		printf("shift   ");
-	if (x & (1 << KG_ALTGR))
-		printf("altgr   ");
-	if (x & (1 << KG_CTRL))
-		printf("control ");
-	if (x & (1 << KG_ALT))
-		printf("alt     ");
+        int t;
+
+	for (t = 0; t < sizeof(modifiers)/sizeof(modifiers[0]); t++)
+	  if (x & (1 << modifiers[t].bit))
+	    printf("%s\t", modifiers[t].name);
 }
 
-void print_bind(int buf[], int i, int j, char numeric) {
+void print_bind(int bufj, int i, int j, char numeric) {
 	printf("\t");
 	print_mod(j);
 	printf("keycode %3d =", i);
-	print_keysym(buf[j], numeric);
+	print_keysym(bufj, numeric);
 	printf("\n");
 }
 
 void dump_keys(char full_table, char numeric) {
 	int i, j, k;
-	int buf[NR_KEYMAPS];
-	int vmax;
-	int vlast;
-	int vcount;
-	int count;
+	int buf[MAX_NR_KEYMAPS];
 	int isletter, islatin, isasexpected;
-	int typ0, val0;
+	int typ, val;
 
-	for (i = 0; i < NR_KEYS; i++) {
-		for (j = 0; j < NR_KEYMAPS; j++)
-			buf[j] = get_bind(i, j);
+	get_keymaps();
+	print_keymaps();
+	if (!keymapnr)
+	  return;
+	for (i = 1; i < NR_KEYS; i++) {
+	    for (j = 0; j < keymapnr; j++)
+	      buf[j] = get_bind(i, good_keymap[j]);
 
-		typ0 = KTYP(buf[0]);
-		val0 = KVAL(buf[0]);
-		islatin = (typ0 == KT_LATIN || typ0 == KT_LETTER);
-		isletter = (islatin &&
-			    ((val0 >= 'A' && val0 <= 'Z') ||
-			     (val0 >= 'a' && val0 <= 'z')));
-		isasexpected = 0;
-		if (isletter && NR_KEYMAPS == 16) {
-		    for(j = 8; j < 16; j++)
-		      if(KTYP(buf[j]) != KT_META ||
-			 KVAL(buf[j]) != KVAL(buf[j-8]))
-			goto unexpected;
-		    for(j = 4; j < 8; j++)
-		      if(KTYP(buf[j] != KT_LATIN) ||
-			 KVAL(buf[j] != (val0 & ~96)))
-			goto unexpected;
-		    for(j = 2; j < 4; j++)
-		      if(buf[j] != buf[j-2])
-			goto unexpected;
-		    if(KTYP(buf[1]) != typ0 || KVAL(buf[1]) != (val0 ^ 32))
+	    typ = KTYP(buf[0]);
+	    val = KVAL(buf[0]);
+	    islatin = (typ == KT_LATIN || typ == KT_LETTER);
+	    isletter = (islatin &&
+			((val >= 'A' && val <= 'Z') ||
+			 (val >= 'a' && val <= 'z')));
+	    isasexpected = 0;
+	    if (isletter) {
+		u_short defs[16];
+		defs[0] = K(KT_LETTER, val);
+		defs[1] = K(KT_LETTER, val ^ 32);
+		defs[2] = defs[0];
+		defs[3] = defs[1];
+		for(j=4; j<8; j++)
+		  defs[j] = K(KT_LATIN, val & ~96);
+		for(j=8; j<16; j++)
+		  defs[j] = K(KT_META, KVAL(defs[j-8]));
+
+		for(j = 0; j < keymapnr; j++) {
+		    k = good_keymap[j];
+		    if ((k >= 16 && buf[j] != K_HOLE) || (k < 16 && buf[j] != defs[k]))
 		      goto unexpected;
-		    isasexpected = 1;
 		}
-	      unexpected:
+		isasexpected = 1;
+	    }
+	  unexpected:
 
-		for (j = NR_KEYMAPS - 1; j >= 0; j--)
-			if (buf[j] != K_HOLE)
-				break;
-
-		vmax = j;
-		vlast = j;
-		vcount = 0;
-		for ( ; j > 0; j--) {
-			if (buf[j] != K_HOLE)
-				vcount++;
-			else if (vmax - j + 1 > 2 * vcount)
-				vlast = j - 1;
+	    printf("keycode %3d =", i);
+	    if (full_table) {
+		for (j = 0; j < keymapnr; j++)
+		  print_keysym(buf[j], numeric);
+		printf("\n");
+	    } else
+	    if (isasexpected) {
+		/* print only a single entry */
+		/* suppress the + for ordinary a-zA-Z */
+		print_keysym(K(KT_LATIN, val), numeric);
+		printf("\n");
+	    } else if (!islatin) {
+		/* choose between single entry line followed by exceptions,
+		   and long line followed by exceptions; avoid VoidSymbol */
+		int bad = 0;
+		int count = 0;
+		for(j = 1; j < keymapnr; j++) {
+		    if (buf[j] != buf[0])
+		      bad++;
+		    if (buf[j] != K_HOLE)
+		      count++;
 		}
-		if (vlast == 0 && vmax > 0)
-		        if(isletter || !islatin)
-			        vlast = 1;
-
-		count = 0;
-		for (j = 1; j < NR_KEYMAPS; j++)
-			if (buf[0] != buf[j])
-				count++;
-
-		printf("keycode %3d =", i);
-		if (full_table) {
-		        for (j = 0; j < NR_KEYMAPS; j++)
-			        print_keysym(buf[j], numeric);
-			printf("\n");
-		} else
-		if (!islatin && count < NR_KEYMAPS / 2) {
-			if (buf[0] != K_HOLE)
-				print_keysym(buf[0], numeric);
-			printf("\n");
-			for (j = 1; j < NR_KEYMAPS; j++)
-				if (buf[0] != buf[j])
-					print_bind(buf, i, j, numeric);
-		} else
-		if (isasexpected) {
-		        /* suppress the + for ordinary a-zA-Z */
-			print_keysym(K(KT_LATIN, val0), numeric);
-			printf("\n");
+		if (bad <= count) {
+		    if (buf[0] != K_HOLE)
+		      print_keysym(buf[0], numeric);
+		    printf("\n");
+		    for (j = 1; j < keymapnr; j++)
+		      if (buf[j] != buf[0])
+			print_bind(buf[j], i, good_keymap[j], numeric);
 		} else {
-			for (k = 0; k <= vlast; k++)
-				print_keysym(buf[k], numeric);
-			printf("\n");
-			for (; k <= vmax; k++)
-				if (buf[k] != K_HOLE)
-					print_bind(buf, i, k, numeric);
+		    for (j = 0; j < keymapnr && buf[j] != K_HOLE; j++)
+		      print_keysym(buf[j], numeric);
+		    printf("\n");
+		    for ( ; j < keymapnr; j++)
+		      if (buf[j] != K_HOLE)
+			print_bind(buf[j], i, good_keymap[j], numeric);
 		}
+	    } else {
+		for (j = 0; j < keymapnr && buf[j] != K_HOLE; j++)
+		  print_keysym(buf[j], numeric);
+		printf("\n");
+		for ( ; j < keymapnr; j++)
+		  if (buf[j] != K_HOLE)
+		    print_bind(buf[j], i, good_keymap[j], numeric);
+	    }
 	}
 }
 
@@ -267,18 +331,22 @@ void dump_funcs(void) {
 	struct kbsentry fbuf;
 	char *p;
 
-	for (i = 0; i < NR_FUNC; i++) {
+	for (i = 0; i < MAX_NR_FUNC; i++) {
 		fbuf.kb_func = i;
 		if (ioctl(fd, KDGKBSENT, (unsigned long)&fbuf)) {
-			fprintf(stderr, "KDGKBSENT at index %d: ", i);
-			perror("");
-			exit(1);
+		    if (errno == EINVAL && i > 0) /* an old kernel */
+		      break;
+		    fprintf(stderr, "KDGKBSENT at index %d: ", i);
+		    perror("");
+		    exit(1);
 		}
+		if (!fbuf.kb_string[0])
+		        continue;
 		printf("string %s = \"", syms[KT_FN].table[i]);
 		for (p = fbuf.kb_string; *p; p++) {
 		        if (*p == '"' || *p == '\\') {
 			        putchar('\\'); putchar(*p);
-			} else if (isgraph(*p))
+			} else if (isgraph(*p) || *p == ' ')
 				putchar(*p);
 			else
 				printf("\\%03o", *p);
@@ -290,12 +358,12 @@ void dump_funcs(void) {
 #ifdef KDGKBDIACR
 /* isgraph() does not know about iso-8859; printing the character
    unescaped makes the output easier to check. Maybe this should
-   be an option. */
+   be an option. Use locale? */
 void
 outchar (unsigned char c) {
         printf("'");
         printf((c == '\'' || c == '\\') ? "\\%c"
-	       : (isgraph(c) || c >= 0200) ? "%c"
+	       : (isgraph(c) || c == ' ' || c >= 0200) ? "%c"
 	       : "\\%03o", c);
 	printf("'");
 }
@@ -336,7 +404,7 @@ valid options are:
 	-f --full-table	don't use short-hand notations
 	   --funcs-only	display only the function key strings
 	   --keys-only	display only key bindings
-%s	-c --charset=iso-8859-{1,2,3,4,8}
+%s	-c --charset=iso-8859-{1,2,3,4,7,8}
 			interpret character action codes to be from the
 			specified character set
 ",
@@ -408,7 +476,7 @@ main (int argc, char *argv[]) {
 			case 'c':
 				if (strlen(optarg) != 10 || /* iso-8859-x */
 					strncmp(optarg, "iso-8859-", 9) ||
-					!index("12348", optarg[9]))
+					!index("123478", optarg[9]))
 					usage();
 				charset = optarg[9] - '0';
 				printf("charset \"%s\"\n", optarg);
