@@ -9,7 +9,7 @@
 #include <linux/kd.h>
 void
 usage(){
-    fprintf(stderr, "usage: kbd_mode [-a|-k|-s]\n");
+    fprintf(stderr, "usage: kbd_mode [-a|-u|-k|-s]\n");
     exit(1);
 }
 
@@ -38,6 +38,9 @@ main(int argc, char *argv[]){
 	      case K_XLATE:
 		printf("The keyboard is in the default (ASCII) mode\n");
 		break;
+	      case K_UNICODE:
+		printf("The keyboard is in Unicode (UTF-8) mode\n");
+		break;
 	      default:
 		printf("The keyboard is in some unknown mode\n");
 	    }
@@ -47,7 +50,9 @@ main(int argc, char *argv[]){
 	  usage();
 	if (!strcmp(argv[1], "-a"))
 	  mode = K_XLATE;
-	else if (!strcmp(argv[1], "-s") || !strcmp(argv[1], "-u"))
+	else if (!strcmp(argv[1], "-u"))
+	  mode = K_UNICODE;
+	else if (!strcmp(argv[1], "-s"))
 	  mode = K_RAW;
 	else if (!strcmp(argv[1], "-k"))
 	  mode = K_MEDIUMRAW;
