@@ -165,11 +165,11 @@ static char *latin1_syms[] = {
 	"nobreakspace",
 	"exclamdown",
 	"cent",
-	"sterling",
+	"pound",
 	"currency",
 	"yen",
 	"brokenbar",
-	"section",
+	"paragraph",
 	"diaeresis",
 	"copyright",
 	"ordfeminine",
@@ -305,6 +305,21 @@ char *latin4_syms[] = {
     "", "uogonek", "", "", "", "utilde", "umacron", ""
 };
 
+char *iso_8859_7_syms[] = { /* 160-255 */
+    "", "leftquote", "rightquote", "", "", "", "", "",
+    "", "", "", "", "", "", "", "",
+    "", "", "", "", "accent", "diaeresisaccent", "Alphaaccent", "",
+    "Epsilonaccent", "Etaaccent", "Iotaaccent", "rightanglequote", "Omicronaccent", "onehalf", "Upsilonaccent", "Omegaaccent",
+    "iotadiaeresisaccent", "Alpha", "Beta", "Gamma", "Delta", "Epsilon", "Zeta", "Eta",
+    "Theta", "Iota", "Kappa", "Lamda" /*sic*/, "Mu", "Nu", "Ksi", "Omicron",
+    "Pi", "Rho", "", "Sigma", "Tau", "Upsilon", "Phi", "Khi",
+    "Psi", "Omega", "Iotadiaeresis", "Upsilondiaeresis", "alphaaccent", "epsilonaccent", "etaaccent", "iotaaccent",
+    "upsilondiaeresisaccent", "alpha", "beta", "gamma", "delta", "epsilon", "zeta", "eta",
+    "theta", "iota", "kappa", "lamda" /*sic*/, "mu", "nu", "ksi", "omicron",
+    "pi", "rho", "terminalsigma", "sigma", "tau", "upsilon", "phi", "khi",
+    "psi", "omega", "iotadiaeresis", "upsilondiaeresis", "omicronaccent", "upsilonaccent", "omegaaccent", ""
+};
+
 char *iso_8859_8_syms[] = {
     "", "", "", "", "", "", "", "",
     "", "", "multiplication", "", "", "", "", "overscore",
@@ -341,12 +356,12 @@ static char *fn_syms[] = {
 	"F18",
 	"F19",
 	"F20",
-	"Find",
+	"Find",			/* also called: "Home" */
 	"Insert",
 	"Remove",
-	"Select",
-	"Prior",
-	"Next",
+	"Select",		/* also called: "End" */
+	"Prior",		/* also called: "PageUp" */
+	"Next",			/* also called: "PageDown" */
 	"Macro",
 	"Help",
 	"Do",
@@ -357,6 +372,226 @@ static char *fn_syms[] = {
 	"F24",
 	"F25",
 	"F26",
+	"F27",
+	"F28",
+	"F29",
+	"F30",
+	"F31",
+	"F32",
+	"F33",
+	"F34",
+	"F35",
+	"F36",
+	"F37",
+	"F38",
+	"F39",
+	"F40",
+	"F41",
+	"F42",
+	"F43",
+	"F44",
+	"F45",
+	"F46",
+	"F47",
+	"F48",
+	"F49",
+	"F50",
+	"F51",
+	"F52",
+	"F53",
+	"F54",
+	"F55",
+	"F56",
+	"F57",
+	"F58",
+	"F59",
+	"F60",
+	"F61",
+	"F62",
+	"F63",
+	"F64",
+	"F65",
+	"F66",
+	"F67",
+	"F68",
+	"F69",
+	"F70",
+	"F71",
+	"F72",
+	"F73",
+	"F74",
+	"F75",
+	"F76",
+	"F77",
+	"F78",
+	"F79",
+	"F80",
+	"F81",
+	"F82",
+	"F83",
+	"F84",
+	"F85",
+	"F86",
+	"F87",
+	"F88",
+	"F89",
+	"F90",
+	"F91",
+	"F92",
+	"F93",
+	"F94",
+	"F95",
+	"F96",
+	"F97",
+	"F98",
+	"F99",
+	"F100",
+	"F101",
+	"F102",
+	"F103",
+	"F104",
+	"F105",
+	"F106",
+	"F107",
+	"F108",
+	"F109",
+	"F110",
+	"F111",
+	"F112",
+	"F113",
+	"F114",
+	"F115",
+	"F116",
+	"F117",
+	"F118",
+	"F119",
+	"F120",
+	"F121",
+	"F122",
+	"F123",
+	"F124",
+	"F125",
+	"F126",
+	"F127",
+	"F128",
+	"F129",
+	"F130",
+	"F131",
+	"F132",
+	"F133",
+	"F134",
+	"F135",
+	"F136",
+	"F137",
+	"F138",
+	"F139",
+	"F140",
+	"F141",
+	"F142",
+	"F143",
+	"F144",
+	"F145",
+	"F146",
+	"F147",
+	"F148",
+	"F149",
+	"F150",
+	"F151",
+	"F152",
+	"F153",
+	"F154",
+	"F155",
+	"F156",
+	"F157",
+	"F158",
+	"F159",
+	"F160",
+	"F161",
+	"F162",
+	"F163",
+	"F164",
+	"F165",
+	"F166",
+	"F167",
+	"F168",
+	"F169",
+	"F170",
+	"F171",
+	"F172",
+	"F173",
+	"F174",
+	"F175",
+	"F176",
+	"F177",
+	"F178",
+	"F179",
+	"F180",
+	"F181",
+	"F182",
+	"F183",
+	"F184",
+	"F185",
+	"F186",
+	"F187",
+	"F188",
+	"F189",
+	"F190",
+	"F191",
+	"F192",
+	"F193",
+	"F194",
+	"F195",
+	"F196",
+	"F197",
+	"F198",
+	"F199",
+	"F200",
+	"F201",
+	"F202",
+	"F203",
+	"F204",
+	"F205",
+	"F206",
+	"F207",
+	"F208",
+	"F209",
+	"F210",
+	"F211",
+	"F212",
+	"F213",
+	"F214",
+	"F215",
+	"F216",
+	"F217",
+	"F218",
+	"F219",
+	"F220",
+	"F221",
+	"F222",
+	"F223",
+	"F224",
+	"F225",
+	"F226",
+	"F227",
+	"F228",
+	"F229",
+	"F230",
+	"F231",
+	"F232",
+	"F233",
+	"F234",
+	"F235",
+	"F236",
+	"F237",
+	"F238",
+	"F239",
+	"F240",
+	"F241",
+	"F242",
+	"F243",
+	"F244",
+	"F245",
+	"F246"		/* there are 10 keys named Insert etc., total 256 */
 };
 
 static char *spec_syms[] = {
@@ -374,7 +609,10 @@ static char *spec_syms[] = {
 	"Scroll_Backward",
 	"Boot",
 	"Caps_On",
-	"Compose"
+	"Compose",
+	"SAK",
+	"Decr_Console",
+	"Incr_Console"
 };
 
 static char *pad_syms[] = {
@@ -445,7 +683,11 @@ static char *shift_syms[] = {
 	"AltGr",
 	"Control",
 	"Alt",
-	"Uncaps_Shift"
+	"ShiftL",
+	"ShiftR",
+	"CtrlL",
+	"CtrlR",
+	"CapsShift"
 };
 
 static char *meta_syms[] = {
@@ -589,14 +831,34 @@ static char *ascii_syms[] = {
 	"Ascii_6",
 	"Ascii_7",
 	"Ascii_8",
-	"Ascii_9"
+	"Ascii_9",
+	"Hex_0",
+	"Hex_1",
+	"Hex_2",
+	"Hex_3",
+	"Hex_4",
+	"Hex_5",
+	"Hex_6",
+	"Hex_7",
+	"Hex_8",
+	"Hex_9",
+	"Hex_A",
+	"Hex_B",
+	"Hex_C",
+	"Hex_D",
+	"Hex_E",
+	"Hex_F"
 };
 
 static char *lock_syms[] = {
 	"Shift_Lock",
 	"AltGr_Lock",
 	"Control_Lock",
-	"Alt_Lock"
+	"Alt_Lock",
+	"ShiftL_Lock",
+	"ShiftR_Lock",
+	"CtrlL_Lock",
+	"CtrlR_Lock"
 };
 
 #define E(x) { x, sizeof(x) / sizeof(char **) }
@@ -632,6 +894,24 @@ synonyms[] = {
 	{ "PageUp",		"Prior" },
 	{ "PageDown",		"Next" },
 	{ "multiply",		"multiplication" },
+	{ "sterling",		"pound" },
+	{ "section",		"paragraph" },
+	{ "Shift_L",		"ShiftL" },
+	{ "Shift_R",		"ShiftR" },
+	{ "Control_L",		"CtrlL" },
+	{ "Control_R",		"CtrlR" },
+	{ "Alt_L",		"Alt" },
+	{ "Alt_R",		"AltGr" },
+	{ "AltGr_L",		"Alt" },
+	{ "AltGr_R",		"AltGr" },
+/* the names of the Greek letters are spelled differently
+   in the iso-8859-7 and the Unicode standards */
+	{ "lambda",             "lamda" },
+	{ "Lambda",             "Lamda" },
+	{ "xi",                 "ksi" },
+	{ "Xi",                 "Ksi" },
+	{ "chi",                "khi" },
+	{ "Chi",                "Khi" }
 };
 
 const int syms_size = sizeof(syms) / sizeof(syms_entry);
@@ -646,6 +926,7 @@ struct cs {
     { "iso-8859-2", latin2_syms },
     { "iso-8859-3", latin3_syms },
     { "iso-8859-4", latin4_syms },
+    { "iso-8859-7", iso_8859_7_syms },
     { "iso-8859-8", iso_8859_8_syms }
 };
 
@@ -665,7 +946,8 @@ set_charset(char *charset) {
 		return;
 	    }
 	}
-	fprintf (stderr, "unknown charset %s - ignoring charset request\n",
-		 charset);
+	if (!strcmp(charset, "unicode"))
+	  fprintf (stderr, "unknown charset %s - ignoring charset request\n",
+		   charset);
 }
 
