@@ -17,9 +17,15 @@ leave(n) int n; {
   exit(n);
 }
 
-int main () {
+int main (argc,argv) int argc; char **argv; {
   unsigned char i, j, k;
 
+  if (argc != 1) {
+      fprintf(stderr, "usage: showfont
+(probably after loading a font with `setfont font')\n");
+      exit(1);
+  }
+
   if (ioctl(0,GIO_SCRNMAP,obuf)) {
       perror("GIO_SCRNMAP");
       exit(1);
